# -*- coding: utf-8 -*-

from tornado import ioloop
import time
from kola.utils import safe_call


class TimeoutMgr(object):

    timeout = None

    def set(self, interval, callback, repeat=False, force=True):
        """
        添加timeout
        """
        if self.timeout:
            if force:
                # 如果已经存在，那么先要把现在的清空
                self.clear()
            else:
                # 已经存在的话，就返回了
                return

        deadline = time.time() + interval

        def callback_wrapper():
            # 这样可以让timeout及时变成None
            if self.timeout and deadline == self.timeout.deadline:
                # 必须要确定，这次调用就是这个timeout引起的
                self.timeout = None
                # 先执行这里
                result = safe_call(callback)
                if repeat:
                    # 重复的调用
                    self.set(interval, callback, repeat=repeat)
                return result

        self.timeout = ioloop.IOLoop.current().add_timeout(deadline, callback_wrapper)

    def clear(self):
        """
        直接把现在的清空
        """
        if not self.timeout:
            return
        ioloop.IOLoop.current().remove_timeout(self.timeout)
        self.timeout = None
